#root{width:100%;margin:0 auto}.app{height:100vh;width:100%;display:flex;flex-direction:row}.main{display:flex;flex-direction:column;flex:1;overflow:hidden;height:100vh}textarea{padding:.9rem 1rem;border:none;border-radius:.375rem;flex:1;font:inherit;font-size:1rem;line-height:1.5rem;resize:none}textarea:focus{outline:none}form.file-upload button,button.send{border:0;background:#eee;margin:.5rem;border-radius:.5rem;padding:.3rem .5rem;cursor:pointer;height:fit-content}form.file-upload button{margin-right:0}form.file-upload svg,button.send svg{margin-top:3px;width:20px;height:auto}.input-parent{padding:1rem}svg{color:#000}.input-holder{display:flex;justify-items:center;box-shadow:0 0 10px #0000001a;border:rgba(0,0,0,.1) 1px solid;border-radius:.375rem}.input-holder.focused{border:rgba(0,0,0,.3) 1px solid}.sidebar{min-width:260px;background:#202123;height:100%;display:flex;flex-direction:column;justify-content:space-between;padding:.75rem;color:#fff}.logo{text-align:center;font-weight:700;font-size:1.5rem;margin-top:1rem}.github a{color:#999;font-size:.8rem}a:hover{color:#fff}.logo svg{color:#fff;width:35px;height:35px;margin-right:5px;margin-bottom:-8px}.settings{display:flex;flex-direction:column}button,select{padding:.8rem;border:none;border-radius:.25rem;background-color:#333;color:#fff}select option{padding:.8rem}button{cursor:pointer;margin-bottom:.5rem;background:#74a89b}label{font-size:.9rem;color:#999;margin-top:.5rem;margin-bottom:.25rem}label.header{font-size:1.2rem}div.message.system{background:rgba(247,247,248)}div.avatar{background:rgb(194,142,210);padding:5px;color:#fff;border-radius:2px}div.avatar svg{display:block;color:#fff}.chat-messages{flex:1;overflow-y:auto;overflow-x:hidden}div.message.system div.avatar{background:#74a89b}.cell-output{white-space:pre-wrap}div.message{padding:1rem;border-bottom:rgba(0,0,0,.1);flex-direction:row;width:100%;display:flex}div.message a:hover{color:#000;text-decoration:underline}div.message-body{padding-left:1rem;width:100%}.loader{border:5px solid #74a89b;border-radius:50%;border-top:5px solid #457e70;width:20px;height:20px;-webkit-animation:spin 2s linear infinite;animation:spin 2s linear infinite;margin:0 .5rem;margin-bottom:-5px;display:inline-block}@keyframes spin{0%{transform:rotate(0)}to{transform:rotate(360deg)}}:root{font-family:Inter,system-ui,Avenir,Helvetica,Arial,sans-serif;line-height:1.5;font-weight:400;color:#213547;background-color:#fff;font-synthesis:none;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;-webkit-text-size-adjust:100%}a{font-weight:500;color:#000;text-decoration:inherit}*{box-sizing:border-box}body{margin:0;padding:0}h1{font-size:3.2em;line-height:1.1}
